/* [Text content] */
// Font type
font = "Berlin Sans FB Demi";
// Lines of text to write on the coin
lines_text = [ "not", "1", "euro" ];
// Font size of each line (mm)
lines_size = [ 4.0, 8.0, 4.0 ];  // [0:0.1:9999]
// Vertical position of each line from the center (mm)
lines_y = [ 6.5, 0.0, -6.5 ];  // [-9999:0.1:9999]

/* [Size, orientation] */
// Height of the token (mm)
token_height = 2.33;  // [0:0.01:50]
// Total diameter of the coin (mm)
coin_diameter = 23.25;  // [0:0.01:100]
// Flip the model upside-down
flip_upside_down = false;

/* [Miscellaneous] */
// Border radius of the coin (mm)
coin_border_radius = 1.6;  // [0:0.1:50]
// Extrusion height of the text (mm)
coin_text_height = 0.4;  // [0:0.1:50]
// Diameter of the keyring hole (mm)
ring_hole_diameter = 4.5;  // [0:0.1:20]
// Extra distance between coin and keyring hole (mm)
ring_hole_distance = 2;  // [0:0.1:200]
// Radius of the smoothing curve (mm)
smooth_radius = 1;  // [0:0.1:10]

// // 1 euro
// token_height = 2.33;
// coin_diameter = 23.25;
// lines_text = ["not", "1", "euro"];
// lines_size = [4, 8, 4];
// lines_y = [6,5, 0, -6.5];

// // 100 ft
// token_height = 2.6;
// coin_diameter = 23.8;
// lines_text = [ "not", "100", "ft" ];
// lines_size = [ 4, 8, 4 ];
// lines_y = [ 6.5, 0, -6.5 ];

// // 200 ft
// token_height = 2.0;
// coin_diameter = 28.3;
// lines_text = [ "not", "200", "ft" ];
// lines_size = [ 4, 8, 4 ];
// lines_y = [ 7.5, 0, -7.5 ];

module draw_token() {
  coin_radius = coin_diameter / 2;
  ring_hole_radius = ring_hole_diameter / 2;
  coin_inner_radius = coin_radius - coin_border_radius;
  ring_hole_outer_radius = ring_hole_radius * 2;
  ring_hole_y = -coin_radius - ring_hole_radius - ring_hole_distance;
  total_length =
      coin_diameter + ring_hole_distance + ring_hole_outer_radius * 2;
  scale_corr_x = coin_radius / (coin_radius + smooth_radius);
  scale_corr_y = total_length / (total_length + smooth_radius * 2);
  scale_corr_z = token_height / (token_height + smooth_radius * 2);
  delta = 0.01;

  // rounded token base
  difference() {
    scale([ scale_corr_x, scale_corr_y, scale_corr_z ]) {
      translate([ 0, 0, smooth_radius ]) {
        minkowski() {
          hull() {
            // coin base
            cylinder(r = coin_radius, h = token_height);
            // keyring hole base
            translate([ 0, ring_hole_y, 0 ]) {
              cylinder(h = token_height, r = ring_hole_outer_radius);
            }
          }
          // smoothing operator
          sphere(r = smooth_radius);
        }
      }
    }
    // coin text background
    translate([ 0, 0, token_height - coin_text_height ]) {
      cylinder(r = coin_inner_radius, h = coin_text_height + delta);
    }
    // keyring hole
    translate([ 0, ring_hole_y, -delta ]) {
      cylinder(h = token_height + delta * 2, r = ring_hole_radius);
    }
  }

  // coin text
  translate([ 0, 0, token_height - coin_text_height - delta ]) {
    linear_extrude(coin_text_height + delta) {
      for (i = [0:1:len(lines_text) - 1]) {
        translate([ 0, lines_y[i], 0 ]) {
          text(text = lines_text[i], size = lines_size[i], font = font,  //
               halign = "center", valign = "center");
        }
      }
    }
  }
}

$fn = 75;
if (flip_upside_down) {
  rotate([ 0, 180, 0 ]) translate([ 0, 0, -token_height ]) {
    draw_token();
  }
} else {
  draw_token();
}